# The Paint App
from tkinter import *
from tkinter.ttk import Combobox
from tkinter import colorchooser
from tkinter import filedialog
from PIL import Image, ImageGrab
import tkinter as tk
import io


class PaintApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Paint Application")

        # Initialize drawing variables
        self.drawing = False
        self.last_x, self.last_y = None, None
        self.pen_color = "black"
        self.pen_width = 2

        # Create a canvas widget for drawing
        self.canvas = tk.Canvas(root, width=450, height=300, bg="white")
        self.canvas.pack()

        # Create UI elements
        self.create_pen_width_combobox()
        self.create_action_buttons()
        self.create_color_swatch()

        # Bind mouse events to canvas actions
        self.canvas.bind("<Button-1>", self.start_drawing)
        self.canvas.bind("<ButtonRelease-1>", self.stop_drawing)
        self.canvas.bind("<B1-Motion>", self.draw)  # Pressing and dragging the left mouse button

    def create_pen_width_combobox(self):
        # Create a label for pen width
        pen_width_label = tk.Label(self.root, text="Pen Width")
        pen_width_label.pack(side=tk.LEFT)

        # Create a Combobox for selecting pen width
        self.pen_width_combobox = Combobox(self.root, values=["2", "4", "6", "8", "10"], width=5)
        self.pen_width_combobox.pack(side=tk.LEFT)
        self.pen_width_combobox.set("2")
        # This Python statement binds the select_pen_width method to the <<ComboboxSelected>>
        # event of the pen_width_combobox.
        # So, when an item is selected from the pen_width_combobox, the select_pen_width method will be invoked.

        self.pen_width_combobox.bind("<<ComboboxSelected>>", self.select_pen_width)

    def create_color_swatch(self):
        # Create a label for color swatch
        color_swatch_label = tk.Label(self.root, text="Color Swatch:")
        color_swatch_label.pack(side=tk.LEFT)

        # Create a Canvas widget for displaying color swatch
        self.color_swatch_canvas = tk.Canvas(self.root, width=20, height=20, bg=self.pen_color)
        self.color_swatch_canvas.pack(side=tk.LEFT)

    def create_action_buttons(self):
        # Create button for picking color
        color_button = Button(self.root, text="Pick Color", command=self.pick_color)
        color_button.pack(side=tk.LEFT)

        # Create button for erasing (changing color to white)
        erase_button = Button(self.root, text="Erase", command=lambda c='white': self.change_color(c))
        erase_button.pack(side=tk.LEFT)

        # Create button for clearing the canvas
        clear_button = Button(self.root, text="Clear", command=self.clear_canvas)
        clear_button.pack(side=tk.LEFT)

        # Create the "Save As" button and place it on the right side
        save_button = Button(self.root, text="Save As", command=self.save_as)
        save_button.pack(side=tk.RIGHT)

    def start_drawing(self, event):
        # Start drawing when the left mouse button is pressed
        self.drawing = True
        self.last_x, self.last_y = event.x, event.y

    def stop_drawing(self, event):
        # Stop drawing when the left mouse button is released
        self.drawing = False

    # See explanantion below
    def draw(self, event):
        # Draw lines while the left mouse button is held down
        if self.drawing:
            x, y = event.x, event.y
            self.canvas.create_line((self.last_x, self.last_y, x, y), fill=self.pen_color, width=self.pen_width)
            self.last_x, self.last_y = x, y

    def change_color(self, new_color):
        # Change the drawing color
        self.pen_color = new_color
        self.update_color_swatch()

    def pick_color(self):
        # Open a color chooser dialog and set the drawing color
        color = colorchooser.askcolor(initialcolor=self.pen_color)[1]
        if color:
            self.pen_color = color
            self.update_color_swatch()

    def select_pen_width(self, event):
        # Set the pen width based on the selected value in the Combobox
        selected_width = int(self.pen_width_combobox.get())
        self.set_pen_width(selected_width)

    def set_pen_width(self, width):
        # Set the pen width
        self.pen_width = width
        self.update_cursor()

    def clear_canvas(self):
        # Clear the canvas
        self.canvas.delete('all')

    def save_as(self):
        # Open a file dialog for saving the drawing as an image
        file_path = filedialog.asksaveasfilename(defaultextension=".png",
                                                 filetypes=[("PNG files", "*.png"), ("All files", "*.*")])

        if file_path:
            # Use ImageGrab to capture the canvas content
            x, y, w, h = self.canvas.winfo_rootx(), self.canvas.winfo_rooty(), self.canvas.winfo_reqwidth(), self.canvas.winfo_reqheight()
            image = ImageGrab.grab(bbox=(x, y, x + w, y + h))

            # Save the image as PNG
            image.save(file_path)
            print(f"Drawing saved to: {file_path}")

    def update_color_swatch(self):
        # Update the color swatch display
        self.color_swatch_canvas.config(bg=self.pen_color)

    def update_cursor(self):
        # Update the cursor shape based on the pen width
        cursor_size = int(self.pen_width)
        cursor_size = min(max(cursor_size, 1), 16)  # Ensure cursor size is between 1 and 16
        cursor_shape = f"dot {cursor_size}"
        self.root.config(cursor=cursor_shape)


# Main program
if __name__ == "__main__":
    root = tk.Tk()
    paint_app = PaintApp(root)
    root.mainloop()
